function handle=pcolorbar1(crange,cint)
% PCOLORBAR1 (crange,cint,'vert'/'hori') Display color bar (color scale).
%
%   ----------
%		crange: a colume or row vector (this color range should be same as the range for caxis,
%													in order to get a correct color bar.)
%		cint:	  a number of total color interval
%   ----------
%
%   PCOLORBAR1('vert') appends a vertical color scale to the current
%   axis. PCOLORBAR('horiz') appends a horizontal color scale.
%
%   PCOLORBAR(H) places the colorbar in the axes H. The colorbar will
%   be horizontal if the axes H width > height (in pixels).
%
%   PCOLORBAR without arguments either adds a new vertical color scale
%   or updates an existing colorbar.
%
%   H = PCOLORBAR(...) returns a handle to the colorbar axis.

%   Clay M. Thompson 10-9-92
%   Copyright (c) 1984-97 by The MathWorks, Inc.
%   $Revision: 5.21 $  $Date: 1997/04/08 06:10:08 $

%   If called with COLORBAR(H) or for an existing colorbar, don't change
%   the NextPlot property.
changeNextPlot = 1;

if (nargin == 1 | nargin == 2), loc = 'vert'; end
ax = [];
if nargin >= 3, loc = 'hori'; end

% Determine color limits by context.  If any axes child is an image
% use scale based on size of colormap, otherwise use current CAXIS.

ch = get(gca,'children');
hasimage = 0; t = [];
cdatamapping = 'direct';

for i=1:length(ch),
    typ = get(ch(i),'type');
    if strcmp(typ,'image'),
        hasimage = 1;
        cdatamapping = get(ch(i), 'CDataMapping');
    elseif strcmp(typ,'surface') & ...
            strcmp(get(ch(i),'FaceColor'),'texturemap') % Texturemapped surf
        hasimage = 2;
        cdatamapping = get(ch(i), 'CDataMapping');
    elseif strcmp(typ,'patch') | strcmp(typ,'surface')
        cdatamapping = get(ch(i), 'CDataMapping');
    end
end

if ( strcmp(cdatamapping, 'scaled') )
    if hasimage,
        if isempty(t); 
            t = caxis; 
        end
    else
        t = caxis;
        d = (t(2) - t(1))/size(colormap,1);
        t = [t(1)+d/2  t(2)-d/2];
    end
else
    if hasimage,
        t = [1, size(colormap,1)]; 
    else
        t = [1.5  size(colormap,1)+.5];
    end
end

h = gca;

if nargin==0,
    % Search for existing colorbar
    ch = get(gcf,'children'); ax = [];
    for i=1:length(ch),
        d = get(ch(i),'userdata');
        if prod(size(d))==1 & isequal(d,h), 
            ax = ch(i); 
            pos = get(ch(i),'Position');
            if pos(3)<pos(4), loc = 'vert'; else loc = 'horiz'; end
            changeNextPlot = 0;
            break; 
        end
    end
end

origNextPlot = get(gcf,'NextPlot');
if strcmp(origNextPlot,'replacechildren') | strcmp(origNextPlot,'replace'),
    set(gcf,'NextPlot','add')
end

if loc(1)=='v', % Append vertical scale to right of current plot
    
    if isempty(ax),
        units = get(h,'units'); set(h,'units','normalized')
        pos = get(h,'Position'); 
        [az,el] = view;
        stripe = 0.03; edge = 0.035; 
        if all([az,el]==[0 90]), space = 0.03; else space = .1; end
        set(h,'Position',[pos(1) pos(2) pos(3) pos(4)])
        rect = [pos(1)+(1-stripe+edge)*pos(3) pos(2) stripe*pos(3) pos(4)];
        %rect = [pos(1)+(1-stripe-edge)*pos(3) pos(2) pos(3)*stripe pos(4)];
        
        % Create axes for stripe
        ax = axes('Position', rect);
        set(h,'units',units)
    else
        axes(ax);
    end
    
    % Create color stripe
    n = size(colormap,1);
%   image([0 1],t,(1:n)','Tag','TMW_COLORBAR'); set(ax,'Ydir','normal')

% following two options for plotting are fine
% 1.
%	X = [1:2]; Y = [crange(1):cint:crange(2)+cint]; 
%	pcolor(X,Y,[Y(:) Y(:)]); 

% 2.
    X = [1:2]; Y = [crange(1):cint:crange(2)]; 
    cpcolor(X,Y,[Y(:) Y(:)]); 
    %shading flat

    set(ax,'YAxisLocation','right'); % shading flat
    set(ax,'xtick',[],'ytick',Y(1:2:end),'yticklabel',Y(1:2:end))
    set(ax,'xlim',[1.5 2]);
       
elseif loc(1)=='h', % Append horizontal scale to top of current plot
    
    if isempty(ax),
        units = get(h,'units'); set(h,'units','normalized')
        pos = get(h,'Position');
        stripe = 0.06; space = 0.1;
        set(h,'Position',...
            [pos(1) pos(2)+(stripe+space)*pos(4) pos(3) (1-stripe-space)*pos(4)])
        rect = [pos(1) pos(2) pos(3) stripe*pos(4)];
        
        % Create axes for stripe
        ax = axes('Position', rect);
        set(h,'units',units)
    else
        axes(ax);
    end
    
    % Create color stripe
%    X = [1:2]; Y = [crange(1):cint:crange(2)]; 
%    pcolor(X,Y,[Y(:) Y(:)]);
% 1.    X = [1:2]; Y = [crange(1):cint:crange(2)+cint]; 
%    pcolor(X,Y,[Y(:) Y(:)]);

% 2.    
	X = [1:2]; Y = [crange(1):cint:crange(2)]; 
   cpcolor(X,Y,[Y(:) Y(:)]);

    set(ax,'ytick',[]); % shading flat
    set(ax,'ygrid','off');
else
  error('COLORBAR expects a handle, ''vert'', or ''horiz'' as input.')
end
set(ax,'userdata',h)
set(gcf,'CurrentAxes',h)
if changeNextPlot
    set(gcf,'Nextplot','ReplaceChildren')
else
    set(gcf,'NextPlot',origNextPlot)
end

if nargout>0, handle = ax; end

