% Vsect_track2.m
%		read in a set of HWRF-HYCOM grib files, and
%       produce a set of the SPEED section along a hurricane track. 
%
%-------
%	input
%-------
%			*.grb   - GRIB files 
%           nhc_Rita05.dat        - NHC disseminated hurricane information
%
%-------
% output
%-------
%           find temperature section on the hurricane footprint
%
%--------
% history
%--------
%   1/28/2009 Modified "Tsect_track2.m" to plot a speed section 
%       along a track. -hsk
%   2/25/2008 Modified 'Tsect_track.m' to read in a set of grib files,
%       instead of *.mat. -hsk
%   11/27/2007 Edited by Hyun-Sook Kim
%------------------------------------------------------------------------
for ncyc=18:6:132
    %ncyc=42;			% ncyc

isave=1;            % logic to save a plot
tplot=0;            % temperature plot for superimposing on density section 
                    % contour plot
tsample=6;  % temporal sampling intervals
                    
%-------------------------------------------  Run information -------------
fcst='NAOMIex';               % model system ID
domain='west';              % hurricane domain
ID='09';
year=2008;                  % year for interest of hurricane
mm=08;
dd=25;
hr=12;
Hname='fGustav08';             % Hurricane name

%-------------------------------------------------------- for users: ------
%  Number of cycle
% ncyc=60;

%------
dn0=datenum(year,mm,dd)+hr/24;
yearmmddhh=datestr(dn0,'yyyymmddHH');
nds=datestr(dn0+ncyc/24,'ddHH');

runID=['cyc',num2str(ncyc,'%03d'),'-',nds];      % cycle ID: e.g., cyc102-2918

disp(' ..... ')
disp(['         Working on Cycle ',runID])
disp(' ')

%--- Data directory
ddir=['D:\NAOMIex\',Hname,'\',yearmmddhh,'\',runID,'\ocean\'];       % run output directory

%--- output directory
% a folder for figure deposit 
odir=['D:\NAOMIex\',Hname,'\',yearmmddhh,'\',runID,'\Vsect_track2\'];     % directory for figures
disp(' ')
mdir=dir([odir]);
if length(mdir)==0
    [o1,o2,o3]=mkdir(odir);
    if o1==1
        disp(['     succesful to create ',odir,' directory'])
    end
else
    disp(['    the ',odir,' directory exists already!'])
end
clear mdir

%--- output data filename
% ofname=[fcst,'-',Hname,'-SST','.dat'];      % ASCII data filename for MLD

%--- setup date for data extraction
%  6-hour data - 
Dr=dir([ddir,'*.grb']);
dn=[];
if length(Dr)==0
    disp([' ... No Data in ',ddir,'...'])
else
  dn=[];
  gfiles=[];
  N=0;
  for K=1:length(Dr)
      ftmp=Dr(K).name;
      shr=ftmp(end-5:end-4);
      if mod(str2num(shr),tsample)==0; 
          N=N+1;
          syd=ftmp(end-9:end-7);
          gfiles{N}=Dr(K).name;
          gd=greg2((str2num(syd)-1)+str2num(shr)/24,year);
          dn(N)=datenum(gd);
          clear gd 
      end
  end
end

%------------------------------------------------------------------
% Ancilary
%-----------
% hurricane domain: grid information
switch lower(domain)
    case {'west'}
        M=load(['ncep12_01_hur_west_grid.mat']);
        hlon=M.hlon; hlat=M.hlat;
    case {'east'}
        M=load(['ncep12_03_hur_east_grid.mat']);
        hlon=M.elon; hlat=M.elat;
end

%--------------------------------------------------------------------
% main section
%    1.Find a set of indicies along the hurricane track
%------------------------------------------------------
nlayers=15;

%----- Track:
% HWRF or GFS storm track file
frc='HWRF';
switch lower(frc)
    case {'hwrf'}
        dlook=dir([ddir,'\trak.hwrf.atcfunix.*']);
        tfile=dlook.name;
        hsk=caseread([ddir,'\',tfile]);            clear dlook tfile
        hhr=str2num(hsk(:,31:33));
        knt=str2num(hsk(:,65:66));      % reference knot
        % find index for reference wind speed=34kt
        ix=find(knt==34);
        hsk=hsk(ix,:);
        nhx=-1*str2num(hsk(:,42:45))/10;
        nhy=str2num(hsk(:,36:38))/10; 
        hhr=hhr(ix);
        ndn=datenum(hsk(:,9:18),'yyyymmddHH')+hhr/24;
        Rads=str2num(hsk(:,74:95)) *1.8;     % size [nm] to [km]
        Rsize=[];
        for n=1:length(ix)
            tmp=Rads(n,:);
            mn=mean(tmp(tmp~=0));
            Rsize(n)=mn;
            clear tmp mn
        end
        clear Rads hhr B ix jx hsk
    case {'gfs'}
        nhc=load('G:\PROJECT07\OFS\GFStrack\Rita.best.track.gfs.txt');
        hx=nhc(:,3); hy=nhc(:,2);
        ymdh=num2str(nhc(:,1));
        ndn=datenum(ymdh,'yyyymmddHH');
end
%--------------- employ interpolation ---------------
%- build 3-D work space for depth and density fields
hxi=interp(nhx,9);
hyi=interp(nhy,9);

tri=delaunay(hlon,hlat);
Kindx=dsearch(hlon,hlat,tri,hxi,hyi);

% work-space
kdn=find(dn>=min(ndn) & dn<=max(ndn));

Z3d=repmat(nan,[length(kdn),nlayers,length(Kindx)]);
V3d=repmat(nan,[length(kdn),nlayers,length(Kindx)]);

disp(' ')
for kk=1:length(kdn)
    nd=kdn(kk);
    
    gfile=[ddir,gfiles{nd}];
    
    disp(sprintf('... Working on %g/%g',nd,length(dn)))
    disp(['                data: ',gfile])
    
    %--- layer depths
    ztmp=getgrib(gfile,'LAYTH',[1 -1 -1],[nlayers -1 -1]);
    zdat=ztmp.data;                     % [481 x 361 x layers]
    ztmp=getgrib(gfile,'SSHG');
    ssh=ztmp.data;
    clear ztmp
    
    zdat(:,:,1)=zdat(:,:,1)+ssh;
%     zdat=permute(zdat,[3 2 1]);         % [layers x 361 x 481]
    zout=[];
    for k=1:nlayers
        zz=squeeze(zdat(:,:,k));
        zz=zz';                     % [361 x 481]
        zs=zz(:);
        zout=[zout zs];             % [173641 x nlayers]
        clear zz zs
    end
    ssh=ssh;
    ssh=ssh(:);
    hsk=zout;
    hsk(:,1)=ssh;
    hsk(:,2:end)=zout(:,1:end-1);
    Zs=cumsum(hsk,2) + zout/2;            % actual depths by adding thickness each layer
    ztmp=Zs(Kindx,:)';
    ztmp(ztmp>1e+10)=nan;
    Z3d(kk,:,:)=ztmp;
        %clear hsk ztmp zdat zout Zs
               
    %--- velocity
    utmp=getgrib(gfile,'UOGRD',[1 -1 -1],[nlayers -1 -1]);  
    uc=utmp.data; clear utmp;
    vtmp=getgrib(gfile,'VOGRD',[1 -1 -1],[nlayers -1 -1]);  
    vc=vtmp.data; clear vtmp;
    spd=abs(uc+i*vc);    clear uc vc
    
    utmp=getgrib(gfile,'UBARO'); 
    vtmp=getgrib(gfile,'VBARO');
    tmp=abs(utmp.data+i*vtmp.data);
    clear utmp vtmp;
    
    Ds=[];
    for k=1:nlayers
        dd=squeeze(spd(:,:,k))+tmp;
        dd=dd';                     % [361 x 481]
        ds=dd(:);
        Ds=[Ds ds];             % [173641 x nlayers]
        clear dd ds
    end
    dtmp=Ds(Kindx,:)';
    dtmp(dtmp>1e+10)=nan;
    D3d(kk,:,:)=dtmp;
        %clear dtmp ddat Ds
        clear gfile
        
    % find indices at the hurricane eye
    q=find(ndn==dn(nd));
    dr=fnddist(hlon,hlat,nhx(q),nhy(q));
    ks=find(dr <= 150);
    smax=nanmax(Ds(ks,1));
    
    % diagnostic plotting:
    Rall=fnddist(hxi,hyi,hxi(1),hyi(1));
    
figure('position',[73   512   1030   424]);
axes('position',[0.11 0.138 0.85 0.72]);
pcolorhsk(repmat(Rall(:)',15,1)-max(Rall),-ztmp,fliplr(dtmp)); shading flat; 
caxis([0 2]); colormap(jet(20)); hc=colorbar; set(hc,'tickdir','out');
axis([-2000 100 -500 0]);
set(gca,'tickdir','out','fontsize',12);
xlb=get(gca,'xticklabel');
xlb=num2str((-1)*str2num(xlb));
set(gca,'xticklabel',xlb); 
hold on; grid;
dr=fnddist(nhx(q),nhy(q),hxi(1),hyi(1));
plot(-[dr;dr],repmat([-500;0],1,2),'-m','linewidth',1.3);
ylabel('depth (m)');
xlabel(['Distance (km) @(xo,yo)=(',num2str(hxi(1),'%5.2f^oW'),',',num2str(hyi(1),'%5.2f^oN)'),')'])
text(0.05,0.15,[datestr(ndn(q),'yyyy/mm/dd HHZ')],'fontsize',15,'fontweight','bold',...
    'color','r','units','normalized');
text(0.5,0.05,['|V|_{max}=',num2str(smax,'%5.2f'),' m/s (in 150-km Radius)'],'fontsize',11,...
    'fontweight','bold','color','r','units','normalized');
text(0.68,-0.1,pwd,'fontsize',8,'color','r','units','normalized');

ttl{1}='Ocean Current Speed (m/s) along a track';
ttl{2}=['for ',Hname(2:end),' ',runID,' (IC=',yearmmddhh,')'];
title(ttl,'fontsize',12,'fontweight','bold');

set(gcf,'paperpositionmode','auto');
eval(['print -dpng ',odir,'\Vsect_track_',datestr(ndn(q),'yyyymmddHH')]);


    clear Ds smax ks dr
end;            % do-loop at the 6-hr intervals     
    
clear all; close all; pack
end
s
