% grbMLD_R150km.m
%		read in layer and water temperature data from a set of Stand-alone/Coupled HYCOM simulation grib files,
%       and
%       produce a set of the Mixed-Layer-Depth foot prints along a hurricane track. 
%
%-------
%	input
%-------
%			mld*.txt        - mixed layer depth produced by 'get_layers_*.sh'.
%           nhc_Rita05.dat  - NHC disseminated hurricane information
%-------
% output
%-------
%       1. a set of 4 figures:
%               png1name=[fcst,'_',Hname,'_MLD_Htrack'];
%               png2name=[fcst,'_',Hname,'_MLD_Htrack_matrix'];
%               png3name=[fcst,'_',Hname,'_mnMLD_Htrack'];
%               png4name=[fcst,'_',Hname,'_mnMLD_Htrack_matrix'];
%               png5name=[fcst,'_',Hname,'_diffMLD6hr_Htrack_matrix'];
%               png6name=[fcst,'_',Hname,'_diffMLD12hr_Htrack_matrix'];
%
%       2. ascii output for an MLD matrix as function of date (x) and distance (y).
%
%--------
% history
%--------
%       1/30/3008 Edited by Hyun-Sook Kim. 
%------------------------------------------------------------------------

clear all; close all; 

%ncyc=24;

%-------------------------------------------  Run information -------------
for ncyc=54:6:54
    
eta=300;        % Storm size (km)
isave=1;

tsample=6;                  % sampling

%-------------------------------------------  Run information -------------
fcst4t='HyHWRF';               % forecast ID for title
fcst='H3Y2';               % model system ID
domain='hwp33';              % hurricane domain
year=2013;                  % year for interest of hurricane
mm=07;
dd=07;
hr=18;
Hname='Soulik07w';             % Hurricane name

if ncyc <=6
    ftrk='seven07w';
else
    ftrk=lower(Hname);
end
%-------------------------------------------------------- for users: ------
%  Number of cycle
%------
dn0=datenum(year,mm,dd)+hr/24;
yearmmddhh=datestr(dn0,'yyyymmddHH');
dnc=dn0+ncyc/24;
nds=datestr(dnc,'ddHH');

runID=['cyc',num2str(ncyc,'%03d'),'-',nds];      % cycle ID: e.g., cyc102-2918

disp(' ..... ')
disp(['         Working on Cycle ',runID])
disp(' ')

%--- Data directory
ddir=['Z:\',fcst,'\',Hname,'\',yearmmddhh,'\',runID,'\ocean\'];       % run output directory
adir=['Z:\',fcst,'\',Hname,'\atcf\'];       % atcf directory
%--- output directory
% a folder for figure deposit 
odir=['Z:\',fcst,'\',Hname,'\',yearmmddhh,'\',runID,'\trackMLD\'];     % directory for figures
disp(' ')
mdir=dir([odir]);
if length(mdir)==0
    [o1,o2,o3]=mkdir(odir);
    if o1==1
        disp(['     succesful to create ',odir,' directory'])
    end
else
    disp(['             the ',odir,' directory exists already!'])
end
clear mdir

% output data filename
ofname=[fcst,'-',Hname,'-trackMLD','.dat'];      % ASCII data filename for MLD

%--- setup date for data extraction
%  6-hour data - 
Dr=dir([ddir,'*.F*.3d.grb']);
dn=[];
if length(Dr)==0
    disp([' ... No Data in ',ddir,'...'])
end
%------------------------------------------------------------------
% Ancillary
%-----------
% hurricane domain: grid information
[hlon,hlat]=domain4hyhwrfv2(domain);
lonlat=[min(hlon(:))-5 max(hlon(:)) min(hlat(:)) max(hlat(:))];
fpos1=[150 120 780 520];

%----- Track:
% HWRF or GFS storm track file
frc='HWRF';
switch lower(frc)
    case {'hwrf'}
        tfile=([adir,'\',ftrk,'.',datestr(dnc,'yyyymmddHH'),'.trak.hwrf.atcfunix']);
        hsk=caseread(tfile);            clear dlook tfile
        hhr=str2num(hsk(:,31:33));
        knt=str2num(hsk(:,65:66));      % reference knot
        % find index for reference wind speed=34kt
        ix=find(knt==34);
        hsk=hsk(ix,:);
        hx=str2num(hsk(:,41:45))/10;
        hy=str2num(hsk(:,35:38))/10; 
        hhr=hhr(ix);
        hdn=datenum(hsk(:,9:18),'yyyymmddHH')+hhr/24;
        vmax=str2num(hsk(:,49:51));       % max. wind speed (kts)
        Rads=str2num(hsk(:,74:95)) *1.8;     % size [nm] to [km]
        Rsize=[];
        for n=1:length(ix)
            tmp=Rads(n,:);
            mn=mean(tmp(tmp~=0));
            Rsize(n)=mn;
            clear tmp mn
        end
        clear Rads hhr B ix jx hsk
    case {'gfs'}
        nhc=load('G:\PROJECT07\OFS\GFStrack\Rita.best.track.gfs.txt');
        hx=nhc(:,3); hy=nhc(:,2);
        ymdh=num2str(nhc(:,1));
        hdn=datenum(ymdh,'yyyymmddHH');
end

%--- figure output name:
png1name=[odir,fcst,'_',Hname,'_MLD_Htrack150km'];
png2name=[odir,fcst,'_',Hname,'_MLD_Htrack_matrix1_150km'];
png3name=[odir,fcst,'_',Hname,'_MLD_Htrack_matrix2_150km'];
png4name=[odir,fcst,'_',Hname,'_MLD_timeseries_150km'];
png5name=[odir,fcst,'_',Hname,'_diffMLD6hr_Htrack_matrix_150km'];
png6name=[odir,fcst,'_',Hname,'_diffMLD12hr_Htrack_matrix_150km'];

%--------------------------------------------------------------------
% main section
%    1.Find a set of indicies along the hurricane track
%------------------------------------------------------

%====================================================== Assembling data ===
%--- assembling data
% work-space
dn=[dnc:6/24:dnc+5];
knt=min(length(dn),length(hdn));

Im=repmat(nan,1200,length(hdn));       	% collection of indicies each time step
mnMLD=repmat(nan,length(hdn),1);        	% mean MLD on a hurricane footprint                            
MLDall=repmat(nan,[length(hdn),size(hlon)]); % MLD for all 
disp(' ')
m1=repmat(nan,size(hlon));          % dummy matrix
cindx=[];
for K=1:length(dn);
    
    % find indices at the hurricane eye
    q=find(hdn==dn(K));
    if ~isempty(q)
        
        cindx=[cindx; q];
    
        gfile=[ddir,Dr(K).name];
    
    %disp(sprintf('      %g ... Working with %s',K,gfile))

        gtmp1=getgrib(gfile,'MIXHT');   % pot_temperature
        Dt=gtmp1.data;
        Dt(Dt>1e+10)=nan;
        MLDall(K,:,:)=Dt;
        clear gtmp1
    
    % find indices at the hurricane eye
        dr=fnddist(hlon,hlat,hx(q),hy(q));
        ks=find(dr <= eta);
        length(ks)
        Im(1:length(ks),K)=ks(:);
        mnMLD(K)=nanmean(Dt(ks));
    else
        mnMLD(K)=nan;
    end
    clear ks q dr Dt I gtmp gfile
end
Im(Im==0)=nan;      % replace zeros with NaN

mnMLDtrack=repmat(nan,length(hdn),length(hdn));   % a matrix of mean MLDs 
                                % on a Hurricane footprint for the whole time period
for k=1:knt;
    ssm=squeeze(MLDall(k,:,:));                 % a fixed day
    for p=1:length(hdn);                         % x-direction
        a=find(~isnan(Im(:,p)));
        indx=Im(a,p); clear a;
        mnMLDtrack(k,p)=nanmean(ssm(indx));     % mean SST for pre and post storm passage
    end
end

%--------------------------------------------------------------------------
%   Diagnostic Part
%-------------------
[hdn1,ij,jj]=intersect(hdn(:),dn(:));
hx1=hx(ij); hy1=hy(ij); clear ij jj

% color range
crange=[0:10:150];
njet=length(crange)-1;

%--- FIGURE 1: SST on a track
figure('position',fpos1);
axes('position',[0.09 0.10 0.88 0.81]);
draw_wrldcst360('k'); grid; hold on;
for h=1:knt;
    ssm=squeeze(MLDall(h,:,:));
    a=find(~isnan(Im(:,h)));
    
    if ~isempty(a)
    indx=Im(a,h); clear a;
    
    if mod((dn(h)-dn(1)),1)==0
        sss=repmat(nan,size(ssm));
        sss(indx)=ssm(indx);
        [ii,jj]=find(~isnan(sss));
        if (~isempty(ii) && ~isempty(jj))
            ii=[min(ii):max(ii)];
            jj=[min(jj):max(jj)];
            xs=hlon(ii,jj); ys=hlat(ii,jj); tmp=sss(ii,jj);
            pcolorhsk(xs,ys,tmp); shading flat; 
            caxis([crange(1) crange(end)]); 
            clear sss ii jj xs ys tmp
            b=find(hdn==dn(h));
            plot(hx1(h),hy1(h),'or','markersize',6)    
            avg=nanmean(ssm(indx));
            text(hx(h)+Rsize(h)/sqrt(2)/100,hy(h)+Rsize(h)/sqrt(2)/100,...
            [num2str(avg,'%3.1f'),'-m'],...
            'fontsize',11,'color','r','fontweight','bold');
        end
    end
    end
end
axis([lonlat]);
set(gca,'tickdir','out','fontsize',11);
colormap(jet(njet)); 
hc=colorbar; set(hc,'tickdir','out')
% track
plot(hx,hy,'-xk','markersize',5,'linewidth',1.5);
xlabel('Longitude (^oW)');
ylabel('Latitude (^oN)');
title([fcst4t,': ',runID,'  24-hourly MLD for ',upper(Hname),' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');
txt{1}='from';
txt{2}=datestr(dn(1),'yyyy/mm/dd HH:MMZ');
txt{3}=' to ';
txt{4}=datestr(dn(end),'yyyy/mm/dd HH:MMZ');
text(0.05,0.9,txt,'fontsize',13,'color','m','units','normalized','fontweight','bold');

if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png1name]); 
end

%--- FIGURE2: SST on hurricane footprint: 2 Stamp (10x10) Plots
%
% A) indiviual subplot locations for a 10x10 figure
np=10;
pd=0.084;
pys=0.085+pd*[0:np-1];
pxs=0.085+pd*[0:np-1];
[Px,Py]=meshgrid(pxs,pys);
Px=flipud(Px);
Py=flipud(Py);
axs=[Px(:),Py(:),repmat(pd,np*np,2)];        % each figure location

rscale=2;       % box size (1*rscale x 1*rscale)
sscale=10;          % scale for scatter dot 

% plotting
figure('position',[190   141   811   793]);
axes('position',[0.085 0.96 0.84 0.01]);
pcolor([crange;crange]); caxis([crange(1) crange(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:length(crange)],'xticklabel',[crange]); 
title([fcst4t,': ',runID,' MLD for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=1:np;      % y-direction
    ssm=squeeze(MLDall(k,:,:));       % a fixed day
    for p=1:np;         % x-direction
        r=(k-1)*np+p;
        axes('position',axs(r,:));
        if ismember(r,[1:np+1:np*np]); 
            plot([-1 1]*rscale,[1 -1]*rscale,'k'); hold on; 
        end
        a=find(~isnan(Im(:,p)));
        indx=Im(a,p); clear a;
        
        b=find(hdn==dn(p));                     % concurrent time
        sss=nan(size(ssm));
        if isnan(mnMLDtrack(k,p))==1
            plot(hx(b),hy(b),'.'); 
        else
            sss(indx)=ssm(indx);
            [ia,ja]=find(~isnan(sss)); 
            ia=unique(ia); ja=unique(ja);
            pcolorhsk(hlon(ia,ja)-hx(b),hlat(ia,ja)-hy(b),sss(ia,ja)); shading flat;
        end
        %scatter(hlon(indx)-hx(b),hlat(indx)-hy(b),sscale,ssm(indx),'filled');
        caxis([crange(1) crange(end)]); colormap(jet(njet)); 
        set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
            'ticklength',[0.05 0.025])
        set(gca,'xticklabel',[],'yticklabel',[]);
        if mod(p,np)==0; 
            set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]); 
        end
         text(-0.35,0.8*rscale,[num2str(mnMLDtrack(k,p),'%4.1f'),'-m'],...
             'fontsize',9,'color','k','fontweight','bold');
         box on
        if k==1;        % (lat,lon) at time
            ylabel(['(',num2str(-hx(b),'%4.1f'),',',...
                num2str(hy(b),'%4.1f'),')'],'rotation',90,'fontsize',9,...
                'fontweight','bold');
        end
    end
    xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png2name]); 
end

%break

% Stamp figure - 2
% Figure 2:
% B) indiviual subplot locations for a 10x10 figure
figure('position',[190   141   811   793]);
axes('position',[0.085 0.96 0.84 0.01]);
pcolor([crange;crange]); caxis([crange(1) crange(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:length(crange)],'xticklabel',[crange]); 
title([fcst4t,': ',runID,' MLD for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=np+1:min(2*np,knt);      % y-direction
    ssm=squeeze(MLDall(k,:,:));       % a fixed day
    for p=np+1:min(2*np,knt);         % x-direction
        r=(k-1-np)*np+(p-np);
        %subplot(10,10,r)
        axes('position',axs(r,:));
        if ismember(r,[1:np+1:np*np]); 
            plot([-1 1]*rscale,[1 -1]*rscale,'k'); hold on; 
        end
        a=find(~isnan(Im(:,p)));
        indx=Im(a,p); clear a;
        
        b=find(hdn==dn(p));                     % concurrent time
        sss=nan(size(ssm));
        if isnan(mnMLDtrack(k,p))==1
            plot(hx(b),hy(b),'.'); 
        else
            sss(indx)=ssm(indx);
            [ia,ja]=find(~isnan(sss));
            ia=unique(ia); ja=unique(ja); 
            pcolorhsk(hlon(ia,ja)-hx(b),hlat(ia,ja)-hy(b),sss(ia,ja)); shading flat;
        end
%         scatter(hlon(indx)-hx(b),hlat(indx)-hy(b),sscale,ssm(indx),'filled');
        caxis([crange(1) crange(end)]); colormap(jet(njet)); 
        set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
            'ticklength',[0.05 0.025])
        set(gca,'xticklabel',[],'yticklabel',[]);
        if mod(p,np)==0; 
            set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]); 
        end
        text(-0.35,0.8*rscale,[num2str(mnMLDtrack(k,p),'%4.1f'),'-m'],...
             'fontsize',9,'color','k','fontweight','bold');
         box on
        if k==np+1;        % (lat,lon) at time
            ylabel(['(',num2str(-hx(b),'%4.1f'),',',...
                num2str(hy(b),'%4.1f'),')'],'rotation',90,'fontsize',9,...
                'fontweight','bold');
        end
    end
    xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png3name]); 
end

%--------------------- stamp figure: end ----------------------------------

%--- FIGURE 4: Time series of mean MLD
hdn=hdn(1:length(dn));

mnMLD=mnMLD(1:knt);
mnMLDtrack=mnMLDtrack(1:knt,1:knt);
C=mnMLDtrack';
mnMLDb=diag(C,-1);  % 6-hr before storm
mnMLDa=diag(C,+1);  % 6-hr after storm

figure('position',[65   190   585   765]);
axes('position',[0.13 0.425 0.77 0.47]);
plot(hdn(2:end),-mnMLDb,'-ok','markerfacecolor','b'); hold on; grid
plot(hdn,-mnMLD,'-ok','markerfacecolor','r');
plot(hdn(1:end-1),-mnMLDa,'-ok','markerfacecolor','c');
set(gca,'xtick',[floor(dn(1)):1:ceil(dn(end))],'xlim',[floor(dn(1)) ceil(dn(end))],...
    'tickdir','out','ylim',[-150 0],'xticklabel',[]); 
set(gca,'yminortick','on','fontsize',12);
ylabel('depth (m)');
title([fcst4t,': ',runID,' Mean MLD for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');
legend('6-hr before','during','6-hr after','location','northoutside','orientation','horizontal');

axes('position',[0.13 0.09 0.77 0.33]);
plot(hdn(2:end),mnMLD(2:end)-mnMLDb,'-ok','markerfacecolor','m'); 
hold on; grid
plot(hdn(1:end-1),mnMLDa-mnMLD(1:end-1),'-ok','markerfacecolor','g');
legend('pre 6-hr','post 6-hr','location','northoutside','orientation','horizontal');
set(gca,'xtick',[floor(dn(1)):1:ceil(dn(end))],'xlim',[floor(dn(1)) ceil(dn(end))],...
    'tickdir','out','xticklabel',[]); 
datetickhsk('x','mm/dd HH','keeplimits','keepticks');
ylabel('\deltaMLD(m)');
xlabel(['Year ',num2str(year)],'fontsize',12)

text(0.65,-0.15,pwd,'fontsize',9,'color','r','units','normalized')

if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png4name]); 
end


%--- FIGURE 5: Hurricane's delta MLD foot-print: Stamp (6x6) Plot
% figure 5:

% build a matrix for subplot positions
ixs=[0.0850+0.084*[0:np-1]]';
iys=[0.53 0.2];
Px=[ixs,repmat(iys(:,1),length(ixs),1),repmat([0.085 0.173],length(ixs),1)];
Py=[ixs,repmat(iys(:,2),length(ixs),1),repmat([0.085 0.173],length(ixs),1)];
Pxy=[Px;Py]; 
clear ixs iys Px Py


%--- 6 hour difference
%--- difference in SST over the past 6 hour 
%crange1=[-50:5:50];
crange1=[-50:5:50];
njet=length(crange1)-1;

dMLDall=repmat(nan,size(MLDall));
tmp=diff(MLDall); 
dMLDall(2:end,:,:)=tmp; 
clear tmp

figure('position',[190   441   811   430]);
axes('position',[0.085 0.80 0.84 0.02]);
pcolor([crange1;crange1]); caxis([crange1(1) crange1(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:length(crange1)],'xticklabel',[crange1]); 
title([fcst4t,': ',runID,' \deltaMLD_{6hr} for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=1:min(2*np,knt);      % y-direction
    mld=squeeze(dMLDall(k,:,:));       % a fixed day
    %subplot(6,6,r)
    axes('position',Pxy(k,:));
    a=find(~isnan(Im(:,k)));
    indx=Im(a,k); clear a;

    b=find(hdn==dn(k));                     % concurrent time
    if ~isempty(b)
        if isnan(nanmean(mld(indx)))==1
            plot(hx(b),hy(b),'.'); 
        else
            sss=nan(size(mld));
            sss(indx)=mld(indx);
            [ia,ja]=find(~isnan(sss)); 
            ia=unique(ia); ja=unique(ja);   
            pcolorhsk(hlon(ia,ja)-hx(b),hlat(ia,ja)-hy(b),sss(ia,ja)); shading flat;
        end
        caxis([crange1(1) crange1(end)]); colormap(jet(njet)); 
        set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
                'ticklength',[0.05 0.025])
        set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                    'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]);     
         text(-0.15,0.8*rscale,[num2str(nanmean(mld(indx)),'%4.1f'),'-m'],...
             'fontsize',9,'fontweight','bold');
         box on
        xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
    end
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png5name]); 
end

% save output
eval(['save mnMLD',num2str(ncyc,'%03g'),'.mat C dn']);

clear all; close all; 
end
%--------------------- stamp figure of \delta MLD: end ----------------------------------
