% trackZ26_R150.m
%		read in layer and water temperature data from a set of 
%       Stand-alone/Coupled HYCOM simulation grib files,
%       and
%       produce a set of the Z26 footprints (size of 150 km) along a hurricane track. 
%
%-------
%	input
%-------
%			mld*.txt        - mixed layer depth produced by 'get_layers_*.sh'.
%           nhc_Rita05.dat  - NHC disseminated hurricane information
%-------
% output
%-------
%       1. a set of 4 figures:
%               png1name=[fcst,'_',Hname,'_Z26_HtrackR'];
%               png2name=[fcst,'_',Hname,'_Z26_HtrackR_matrix'];
%               png3name=[fcst,'_',Hname,'_mnZ26_HtrackR'];
%               png4name=[fcst,'_',Hname,'_mnZ26_Htrack_matrix'];
%               png5name=[fcst,'_',Hname,'_diffZ266hr_HtrackR_matrix'];
%               png6name=[fcst,'_',Hname,'_diffZ2612hr_HtrackR_matrix'];
%
%       2. ascii output for an Z26 matrix as function of date (x) and distance (y).
%
%--------
% history
%--------
%   3/18/2009 Modified 'trackMLD.m' by hsk
%   1/30/2008 Edited by Hyun-Sook Kim. 
%------------------------------------------------------------------------

for ncyc=0:12:0
% cycle
%ncyc=00;        
    
isave=1;
eta=300;		% the footprint radius (km)
tsample=6;                  % sampling

%-------------------------------------------  Run information -------------
fcst='NAOMIex';               % model system ID
domain='west';              % hurricane domain
year=2005;                  % year for interest of hurricane
mm=09;
dd=18;
hr=12;
Hname='gRita05';             % Hurricane name

%-------------------------------------------------------- for users: ------
%  Number of cycle
% ncyc=60;

%------
dn0=datenum(year,mm,dd)+hr/24;
yearmmddhh=datestr(dn0,'yyyymmddHH');
nds=datestr(dn0+ncyc/24,'ddHH');

runID=['cyc',num2str(ncyc,'%03d'),'-',nds];      % cycle ID: e.g., cyc102-2918

disp(' ..... ')
disp(['         Working on Cycle ',runID])
disp(' ')

%--- Data directory
ddir=['D:\NAOMIex\',Hname,'\',yearmmddhh,'\',runID,'\ocean\'];       % run output directory

%--- output directory
% a folder for figure deposit 
odir=['D:\NAOMIex\',Hname,'\',yearmmddhh,'\',runID,'\trackZ26\'];     % directory for figures
disp(' ')
mdir=dir([odir]);
if length(mdir)==0
    [o1,o2,o3]=mkdir(odir);
    if o1==1
        disp(['     succesful to create ',odir,' directory'])
    end
else
    disp(['             the ',odir,' directory exists already!'])
end
clear mdir

% output data filename
ofname=[fcst,'-',Hname,'-trackZ26','.dat'];      % ASCII data filename for Z26

%--- setup date for data extraction
%  6-hour data - 
Dr=dir([ddir,'mld*.txt']);
dn=[];
if length(Dr)==0
    disp([' ... No Data in ',ddir,'...'])
else
  dn=[];
  gfiles=[];
  N=0;
  for K=1:length(Dr)
      ftmp=Dr(K).name;
      shr=ftmp(end-5:end-4);
      if mod(str2num(shr),6)==0; 
          N=N+1;
          syd=ftmp(end-9:end-7);
          gfiles{N}=Dr(K).name;
          gd=greg2((str2num(syd)-1)+str2num(shr)/24,year);
          dn(N)=datenum(gd);
          clear gd syd
      end
      clear ftmp shr
  end
end

%------------------------------------------------------------------
% Ancillary
%-----------
% hurricane domain: grid information
switch lower(domain)
    case {'west'}
        M=load(['ncep12_01_hur_west_grid.mat']);
        hlon=M.hlon; hlat=M.hlat;
        lonlat=[-100 -60 15 40];
    case {'mid'}
        M=load(['ncep12_02_hur_mid_grid.mat']);
        hlon=M.mlon; hlat=M.mlat;
        lonlat=[-90 -35 10 45];
    case {'east'}
        M=load(['ncep12_03_hur_east_grid.mat']);
        hlon=M.elon; hlat=M.elat;
        lonlat=[-90 -40 5 50];
end
clear M

load ClomClam;     % coastline

%----- Track:
% HWRF or GFS storm track file
frc='HWRF';
switch lower(frc)
    case {'hwrf'}
        dlook=dir([ddir,'\trak.hwrf.atcfunix.*']);
        tfile=dlook.name;
        hsk=caseread([ddir,'\',tfile]);            clear dlook tfile
        hhr=str2num(hsk(:,31:33));
        knt=str2num(hsk(:,65:66));      % reference knot
        % find index for reference wind speed=34kt
        ix=find(knt==34);
        hsk=hsk(ix,:);
        hx=-1*str2num(hsk(:,42:45))/10;
        hy=str2num(hsk(:,36:38))/10; 
        hhr=hhr(ix);
        hdn=datenum(hsk(:,9:18),'yyyymmddHH')+hhr/24;
        vmax=str2num(hsk(:,49:51));       % max. wind speed (kts)
        Rads=str2num(hsk(:,74:95)) *1.8;     % size [nm] to [km]
        Rsize=[];
        for n=1:length(ix)
            tmp=Rads(n,:);
            mn=mean(tmp(tmp~=0));
            Rsize(n)=mn;
            clear tmp mn
        end
        clear Rads hhr B ix jx hsk
    case {'gfs'}
        nhc=load('G:\PROJECT07\OFS\GFStrack\Rita.best.track.gfs.txt');
        hx=nhc(:,3); hy=nhc(:,2);
        ymdh=num2str(nhc(:,1));
        hdn=datenum(ymdh,'yyyymmddHH');
end
hdni=interp1(hdn,hdn,dn,'linear');

% %nhc=load('G:/NHC/nhc_Rita05.dat');           % NHC hurricane information
% %
% %   1    2   3  4  5   6   7       8         9        10   
% %  ----------------------------------------------------------
% %   year mon dy hh lat lon Prs(mb) wind(m/s) size(km) Prs_env
% %
% nhc=load(['G:/NHC/nhc_',Hname,'.dat']);           % NHC hurricane information
% hx=nhc(:,6); hy=nhc(:,5);
% hdn=datenum([nhc(:,1),nhc(:,2),nhc(:,3),nhc(:,4),repmat(0,length(hx),2)]);
% Rsize=nhc(:,end-1);           % the hurricane radius (km)

%--- figure output name:
png1name=[odir,fcst,'_',Hname,'_Z26_HtrackR'];
png2name=[odir,fcst,'_',Hname,'_Z26_HtrackR_matrix1'];
png3name=[odir,fcst,'_',Hname,'_Z26_HtrackR_matrix2'];
png4name=[odir,fcst,'_',Hname,'_Z26_timeseriesR'];
png5name=[odir,fcst,'_',Hname,'_diffZ266hr_HtrackR_matrix'];
png6name=[odir,fcst,'_',Hname,'_diffZ2612hr_HtrackR_matrix'];

%--------------------------------------------------------------------
% main section
%    1.Find a set of indicies along the hurricane track
%------------------------------------------------------

%====================================================== Assembling data ===
%--- assembling data
% work-space
Im=repmat(nan,1200,length(dn));       	% collection of indicies each time step
mnZ26=repmat(nan,length(dn),1);        	% mean Z26 on a hurricane footprint                            
Z26all=repmat(nan,[length(dn),size(hlon)]); % Z26 for all 
disp(' ')
m1=repmat(nan,size(hlon));          % dummy matrix
for K=1:min(length(dn),length(hdn));
    
    gfile=[ddir,gfiles{K}];
    
    disp(sprintf('      %g ... Working with %s',K,gfile))

    gtmp=load(gfile);

    Dt=m1;
    I=gtmp(:,1);
    Dt(I)=gtmp(:,6);        % Z26
    Dt(Dt>1e+10)=nan;
    
    Z26all(K,:,:)=Dt;
        
    % find indices at the hurricane eye
    q=find(hdn==dn(K));
    dr=fnddist(hlon,hlat,hx(q),hy(q));
    %ks=find(dr <= Rsize(q));
    ks=find(dr <= eta);
    
    length(ks)
    
    Im(1:length(ks),K)=ks(:);
    
    mnZ26(K)=nanmean(Dt(ks));
    
    clear ks q dr Dt I gtmp gfile
end
Im(Im==0)=nan;      % replace zeros with NaN

mnZ26track=repmat(nan,length(dn),length(dn));   % a matrix of mean Z26's 
                                % on a Hurricane footprint for the whole time period
for k=1:min(length(dn),length(hdn));
    ssm=squeeze(Z26all(k,:,:));                 % a fixed day
    for p=1:length(dn);                         % x-direction
        a=find(~isnan(Im(:,p)));
        indx=Im(a,p); clear a;
        mnZ26track(k,p)=nanmean(ssm(indx));     % mean SST for pre and post storm passage
    end
end

%--------------------------------------------------------------------------
%   Diagnostic Part
%-------------------
% color range
crange=[0:10:350];
njet=length(crange)-1;

%--- FIGURE 1: SST on a track
figure('position',[300   400   833   515]);
axes('position',[0.09 0.10 0.88 0.81]);
plot(Clom,Clam,'k'); grid; hold on;
for h=1:min(length(dn),length(hdn));
    ssm=squeeze(Z26all(h,:,:));
    a=find(~isnan(Im(:,h)));
    indx=Im(a,h); clear a;
    if mod((dn(h)-dn(1)),1)==0
        scatter(hlon(indx),hlat(indx),20,ssm(indx),'filled');
        caxis([crange(1) crange(end)]); 
        b=find(hdn==dn(h));
        plot(hx(b),hy(b),'or','markersize',6)    
        avg=nanmean(ssm(indx));
        text(hx(h)+150/sqrt(2)/150,hy(h)+100/sqrt(2)/150,...
            [num2str(round(avg),'%3d'),'-m'],...
            'fontsize',11,'color','k','fontweight','bold');
    end
end
axis([lonlat]);
set(gca,'tickdir','out','fontsize',11);
colormap(jet(njet)); 
hc=colorbar; set(hc,'tickdir','out')
% track
plot(hx,hy,'-xk','markersize',5,'linewidth',1.5);
xlabel('Longitude (^oW)');
ylabel('Latitude (^oN)');
title([fcst,': ',runID,'  24-hourly Z26 for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');
txt{1}='from';
txt{2}=datestr(dn(1),'yyyy/mm/dd HH:MMZ');
txt{3}=' to ';
txt{4}=datestr(dn(end),'yyyy/mm/dd HH:MMZ');
text(0.05,0.9,txt,'fontsize',13,'color','m','units','normalized','fontweight','bold');

if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png1name]); 
end

%--- FIGURE2: SST on hurricane footprint: 2 Stamp (10x10) Plots
%
% A) indiviual subplot locations for a 10x10 figure
np=10;
pd=0.084;
pys=0.085+pd*[0:np-1];
pxs=0.085+pd*[0:np-1];
[Px,Py]=meshgrid(pxs,pys);
Px=flipud(Px);
Py=flipud(Py);
axs=[Px(:),Py(:),repmat(pd,np*np,2)];        % each figure location

rscale=2;       % box size (1*rscale x 1*rscale)
sscale=20;          % scale for scatter dot 

% plotting
figure('position',[190   141   811   793]);
axes('position',[0.085 0.96 0.84 0.01]);
pcolor([crange;crange]); caxis([crange(1) crange(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:2:length(crange)],'xticklabel',[crange(1:2:end)]); 
title([fcst,': ',runID,' Z26 for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=1:np;      % y-direction
    ssm=squeeze(Z26all(k,:,:));       % a fixed day
    for p=1:np;         % x-direction
        r=(k-1)*np+p;
        %subplot(6,6,r)
        axes('position',axs(r,:));
        if ismember(r,[1:np+1:np*np]); 
            plot([-1 1]*rscale,[1 -1]*rscale,'k'); hold on; 
        end
        a=find(~isnan(Im(:,p)));
        indx=Im(a,p); clear a;
        
        b=find(hdn==dn(p));                     % concurrent time
        scatter(hlon(indx)-hx(b),hlat(indx)-hy(b),sscale,ssm(indx),'filled');
        caxis([crange(1) crange(end)]); colormap(jet(njet)); 
        set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
            'ticklength',[0.05 0.025])
        set(gca,'xticklabel',[],'yticklabel',[]);
        if mod(p,np)==0; 
            set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]); 
        end
         text(-0.35,0.8*rscale,[num2str(mnZ26track(k,p),'%4.1f')],...
             'fontsize',9,'color','k','fontweight','bold');
         box on
        if k==1;        % (lat,lon) at time
            ylabel(['(',num2str(-hx(b),'%4.1f'),',',...
                num2str(hy(b),'%4.1f'),')'],'rotation',90,'fontsize',9,...
                'fontweight','bold');
        end
    end
    xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png2name]); 
end

% Stamp figure - 2
% Figure 2:
% B) indiviual subplot locations for a 10x10 figure
figure('position',[190   141   811   793]);
axes('position',[0.085 0.96 0.84 0.01]);
pcolor([crange;crange]); caxis([crange(1) crange(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:2:length(crange)],'xticklabel',[crange(1:2:end)]); 
title([fcst,': ',runID,' Z26 for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=np+1:min(2*np,length(dn));      % y-direction
    ssm=squeeze(Z26all(k,:,:));       % a fixed day
    for p=np+1:min(2*np,min(length(dn),length(hdn)));         % x-direction
        r=(k-1-np)*np+(p-np);
        %subplot(10,10,r)
        axes('position',axs(r,:));
        if ismember(r,[1:np+1:np*np]); 
            plot([-1 1]*rscale,[1 -1]*rscale,'k'); hold on; 
        end
        a=find(~isnan(Im(:,p)));
        indx=Im(a,p); clear a;
        
        b=find(hdn==dn(p));                     % concurrent time
        scatter(hlon(indx)-hx(b),hlat(indx)-hy(b),sscale,ssm(indx),'filled');
        caxis([crange(1) crange(end)]); colormap(jet(njet)); 
        set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
            'ticklength',[0.05 0.025])
        set(gca,'xticklabel',[],'yticklabel',[]);
        if mod(p,np)==0; 
            set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]); 
        end
        text(-0.35,0.8*rscale,[num2str(mnZ26track(k,p),'%4.1f')],...
             'fontsize',9,'color','k','fontweight','bold');
         box on
        if k==np+1;        % (lat,lon) at time
            ylabel(['(',num2str(-hx(b),'%4.1f'),',',...
                num2str(hy(b),'%4.1f'),')'],'rotation',90,'fontsize',9,...
                'fontweight','bold');
        end
    end
    xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png3name]); 
end


%--------------------- stamp figure: end ----------------------------------

%--- FIGURE 4: Time series of mean Z26
C=mnZ26track';
mnZ26b=diag(C,-1);  % 6-hr before storm
mnZ26a=diag(C,+1);  % 6-hr after storm

figure;
axes('position',[0.13 0.15 0.77 0.75]);
plot(dn(2:end),-mnZ26b,'-ok','markerfacecolor','b'); hold on; grid
plot(dn,-mnZ26,'-ok','markerfacecolor','r');
plot(dn(1:end-1),-mnZ26a,'-ok','markerfacecolor','c');
legend('6-hr before','during','6-hr after','orientation','vertical',4);
set(gca,'xtick',[floor(dn(1)):1:ceil(dn(end))],'xlim',[floor(dn(1)) ceil(dn(end))],...
    'tickdir','out'); 
set(gca,'yminortick','on','fontsize',12);
datetickhsk('x','mm/dd HH','keeplimits','keepticks');
ylabel('depth (m)');
title([fcst,': ',runID,' Mean Z26 for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');
xlabel(['Year ',num2str(year)],'fontsize',12)

%keyboard

if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png4name]); 
end


%--- FIGURE 5: Hurricane's delta Z26 foot-print: Stamp (6x6) Plot
% figure 5:

crange=[-50:5:50];
njet=length(crange)-1;

% build a matrix for subplot positions
ixs=[0.0850+0.084*[0:np-1]]';
iys=[0.53 0.2];
Px=[ixs,repmat(iys(:,1),length(ixs),1),repmat([0.085 0.173],length(ixs),1)];
Py=[ixs,repmat(iys(:,2),length(ixs),1),repmat([0.085 0.173],length(ixs),1)];
Pxy=[Px;Py]; 
clear ixs iys Px Py


%--- 6 hour difference
%--- difference in SST over the past 6 hour 
dZ26all=repmat(nan,size(Z26all));
tmp=diff(Z26all); 
dZ26all(2:end,:,:)=tmp; 
clear tmp

figure('position',[190   441   811   430]);
axes('position',[0.085 0.80 0.84 0.02]);
pcolor([crange;crange]); caxis([crange(1) crange(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:length(crange)],'xticklabel',[crange]); 
title([fcst,': ',runID,' \deltaZ26_{6hr} for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=1:min(2*np,min(length(dn),length(hdn)));      % y-direction
    mld=squeeze(dZ26all(k,:,:));       % a fixed day
    %subplot(6,6,r)
    axes('position',Pxy(k,:));
    a=find(~isnan(Im(:,k)));
    indx=Im(a,k); clear a;

    b=find(hdn==dn(k));                     % concurrent time
    scatter(hlon(indx)-hx(b),hlat(indx)-hy(b),40,mld(indx),'filled');
    caxis([crange(1) crange(end)]); colormap(jet(njet)); 
    set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
            'ticklength',[0.05 0.025])
    set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]);     
     text(-0.5,0.8*rscale,[num2str(nanmean(mld(indx)),'%4.1f'),'-m'],...
         'fontsize',9,'fontweight','bold');
     box on
    xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png5name]); 
end

%--- difference in SST over the past 12-hour 
d2Z26all=repmat(nan,size(Z26all));
tmp=Z26all(3:end,:,:)-Z26all(1:end-2,:,:);
d2Z26all(3:end,:,:)=tmp; 
clear tmp

figure('position',[190   441   811   430]);
axes('position',[0.085 0.80 0.84 0.02]);
pcolor([crange;crange]); caxis([crange(1) crange(end)]); colormap(jet(njet));
set(gca,'yticklabel',[],'xtick',[1:length(crange)],'xticklabel',[crange]); 
title([fcst,': ',runID,' \deltaZ26_{12hr} for ',Hname,' (IC=',yearmmddhh,')'],...
    'fontsize',12,'fontweight','bold');

for k=1:min(2*np,min(length(dn),length(hdn)));      % y-direction
    
    ssm=squeeze(d2Z26all(k,:,:));       % a fixed day
    %subplot(6,6,r)
    axes('position',Pxy(k,:));
    a=find(~isnan(Im(:,k)));
    indx=Im(a,k); clear a;

    b=find(hdn==dn(k));                     % concurrent time
    scatter(hlon(indx)-hx(b),hlat(indx)-hy(b),40,ssm(indx),'filled');
    caxis([crange(1) crange(end)]); colormap(jet(njet)); 
    set(gca,'xlim',[-1 1]*rscale,'ylim',[-1 1]*rscale,'tickdir','out',...
            'ticklength',[0.05 0.025])
    set(gca,'xtick',[-floor(rscale) 0 floor(rscale)],...
                'xticklabel',[-floor(rscale) 0 floor(rscale)],'yticklabel',[]);     
    text(-0.5,0.8*rscale,[num2str(nanmean(ssm(indx)),'%4.1f'),'-m'],...
         'fontsize',9,'fontweight','bold');
    box on
    xlabel(datestr(dn(k),'mm/dd HH'),'fontsize',11,'fontweight','bold')
end
if isave; 
    set(gcf,'paperpositionmode','auto');
    eval(['print -dpng ',png6name]); 
end

% save output
eval(['save mnZ26',num2str(ncyc,'%03g'),'.mat C dn']);

clear all; close all; 
end
%--------------------- stamp figure of \delta SST: end ----------------------------------
